# -*- coding: utf-8 -*-

"""
This module
"""

import attr
import typing

from ..core.model import (
    Property, Resource, Tag, GetAtt, TypeHint, TypeCheck,
)
from ..core.constant import AttrMeta

#--- Property declaration ---

@attr.s
class UserHomeDirectoryMapEntry(Property):
    """
    AWS Object Type = "AWS::Transfer::User.HomeDirectoryMapEntry"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-user-homedirectorymapentry.html

    Property Document:
    
    - ``rp_Entry``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-user-homedirectorymapentry.html#cfn-transfer-user-homedirectorymapentry-entry
    - ``rp_Target``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-user-homedirectorymapentry.html#cfn-transfer-user-homedirectorymapentry-target
    """
    AWS_OBJECT_TYPE = "AWS::Transfer::User.HomeDirectoryMapEntry"
    
    rp_Entry: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "Entry"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-user-homedirectorymapentry.html#cfn-transfer-user-homedirectorymapentry-entry"""
    rp_Target: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "Target"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-user-homedirectorymapentry.html#cfn-transfer-user-homedirectorymapentry-target"""

@attr.s
class ServerProtocolDetails(Property):
    """
    AWS Object Type = "AWS::Transfer::Server.ProtocolDetails"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-protocoldetails.html

    Property Document:
    
    - ``p_PassiveIp``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-protocoldetails.html#cfn-transfer-server-protocoldetails-passiveip
    """
    AWS_OBJECT_TYPE = "AWS::Transfer::Server.ProtocolDetails"
    
    p_PassiveIp: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "PassiveIp"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-protocoldetails.html#cfn-transfer-server-protocoldetails-passiveip"""

@attr.s
class UserSshPublicKey(Property):
    """
    AWS Object Type = "AWS::Transfer::User.SshPublicKey"

    Resource Document: None

    Property Document:
    
    """
    AWS_OBJECT_TYPE = "AWS::Transfer::User.SshPublicKey"
    

@attr.s
class ServerIdentityProviderDetails(Property):
    """
    AWS Object Type = "AWS::Transfer::Server.IdentityProviderDetails"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-identityproviderdetails.html

    Property Document:
    
    - ``p_DirectoryId``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-identityproviderdetails.html#cfn-transfer-server-identityproviderdetails-directoryid
    - ``p_InvocationRole``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-identityproviderdetails.html#cfn-transfer-server-identityproviderdetails-invocationrole
    - ``p_Url``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-identityproviderdetails.html#cfn-transfer-server-identityproviderdetails-url
    """
    AWS_OBJECT_TYPE = "AWS::Transfer::Server.IdentityProviderDetails"
    
    p_DirectoryId: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "DirectoryId"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-identityproviderdetails.html#cfn-transfer-server-identityproviderdetails-directoryid"""
    p_InvocationRole: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "InvocationRole"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-identityproviderdetails.html#cfn-transfer-server-identityproviderdetails-invocationrole"""
    p_Url: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "Url"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-identityproviderdetails.html#cfn-transfer-server-identityproviderdetails-url"""

@attr.s
class ServerEndpointDetails(Property):
    """
    AWS Object Type = "AWS::Transfer::Server.EndpointDetails"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-endpointdetails.html

    Property Document:
    
    - ``p_AddressAllocationIds``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-endpointdetails.html#cfn-transfer-server-endpointdetails-addressallocationids
    - ``p_SecurityGroupIds``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-endpointdetails.html#cfn-transfer-server-endpointdetails-securitygroupids
    - ``p_SubnetIds``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-endpointdetails.html#cfn-transfer-server-endpointdetails-subnetids
    - ``p_VpcEndpointId``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-endpointdetails.html#cfn-transfer-server-endpointdetails-vpcendpointid
    - ``p_VpcId``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-endpointdetails.html#cfn-transfer-server-endpointdetails-vpcid
    """
    AWS_OBJECT_TYPE = "AWS::Transfer::Server.EndpointDetails"
    
    p_AddressAllocationIds: typing.List[TypeHint.intrinsic_str] = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "AddressAllocationIds"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-endpointdetails.html#cfn-transfer-server-endpointdetails-addressallocationids"""
    p_SecurityGroupIds: typing.List[TypeHint.intrinsic_str] = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "SecurityGroupIds"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-endpointdetails.html#cfn-transfer-server-endpointdetails-securitygroupids"""
    p_SubnetIds: typing.List[TypeHint.intrinsic_str] = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "SubnetIds"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-endpointdetails.html#cfn-transfer-server-endpointdetails-subnetids"""
    p_VpcEndpointId: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "VpcEndpointId"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-endpointdetails.html#cfn-transfer-server-endpointdetails-vpcendpointid"""
    p_VpcId: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "VpcId"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-server-endpointdetails.html#cfn-transfer-server-endpointdetails-vpcid"""

@attr.s
class UserPosixProfile(Property):
    """
    AWS Object Type = "AWS::Transfer::User.PosixProfile"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-user-posixprofile.html

    Property Document:
    
    - ``rp_Gid``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-user-posixprofile.html#cfn-transfer-user-posixprofile-gid
    - ``rp_Uid``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-user-posixprofile.html#cfn-transfer-user-posixprofile-uid
    - ``p_SecondaryGids``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-user-posixprofile.html#cfn-transfer-user-posixprofile-secondarygids
    """
    AWS_OBJECT_TYPE = "AWS::Transfer::User.PosixProfile"
    
    rp_Gid: float = attr.ib(
        default=None,
        validator=attr.validators.instance_of(float),
        metadata={AttrMeta.PROPERTY_NAME: "Gid"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-user-posixprofile.html#cfn-transfer-user-posixprofile-gid"""
    rp_Uid: float = attr.ib(
        default=None,
        validator=attr.validators.instance_of(float),
        metadata={AttrMeta.PROPERTY_NAME: "Uid"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-user-posixprofile.html#cfn-transfer-user-posixprofile-uid"""
    p_SecondaryGids: typing.List[float] = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(float), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "SecondaryGids"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-transfer-user-posixprofile.html#cfn-transfer-user-posixprofile-secondarygids"""

@attr.s
class ServerProtocol(Property):
    """
    AWS Object Type = "AWS::Transfer::Server.Protocol"

    Resource Document: None

    Property Document:
    
    """
    AWS_OBJECT_TYPE = "AWS::Transfer::Server.Protocol"
    


#--- Resource declaration ---

@attr.s
class Server(Resource):
    """
    AWS Object Type = "AWS::Transfer::Server"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html

    Property Document:
    
    - ``p_Certificate``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-certificate
    - ``p_Domain``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-domain
    - ``p_EndpointDetails``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-endpointdetails
    - ``p_EndpointType``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-endpointtype
    - ``p_IdentityProviderDetails``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-identityproviderdetails
    - ``p_IdentityProviderType``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-identityprovidertype
    - ``p_LoggingRole``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-loggingrole
    - ``p_ProtocolDetails``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-protocoldetails
    - ``p_Protocols``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-protocols
    - ``p_SecurityPolicyName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-securitypolicyname
    - ``p_Tags``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-tags
    """
    AWS_OBJECT_TYPE = "AWS::Transfer::Server"

    
    p_Certificate: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "Certificate"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-certificate"""
    p_Domain: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "Domain"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-domain"""
    p_EndpointDetails: typing.Union['ServerEndpointDetails', dict] = attr.ib(
        default=None,
        converter=ServerEndpointDetails.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(ServerEndpointDetails)),
        metadata={AttrMeta.PROPERTY_NAME: "EndpointDetails"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-endpointdetails"""
    p_EndpointType: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "EndpointType"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-endpointtype"""
    p_IdentityProviderDetails: typing.Union['ServerIdentityProviderDetails', dict] = attr.ib(
        default=None,
        converter=ServerIdentityProviderDetails.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(ServerIdentityProviderDetails)),
        metadata={AttrMeta.PROPERTY_NAME: "IdentityProviderDetails"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-identityproviderdetails"""
    p_IdentityProviderType: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "IdentityProviderType"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-identityprovidertype"""
    p_LoggingRole: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "LoggingRole"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-loggingrole"""
    p_ProtocolDetails: typing.Union['ServerProtocolDetails', dict] = attr.ib(
        default=None,
        converter=ServerProtocolDetails.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(ServerProtocolDetails)),
        metadata={AttrMeta.PROPERTY_NAME: "ProtocolDetails"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-protocoldetails"""
    p_Protocols: typing.List[typing.Union['ServerProtocol', dict]] = attr.ib(
        default=None,
        converter=ServerProtocol.from_list,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(ServerProtocol), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "Protocols"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-protocols"""
    p_SecurityPolicyName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "SecurityPolicyName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-securitypolicyname"""
    p_Tags: typing.List[typing.Union[Tag, dict]] = attr.ib(
        default=None,
        converter=Tag.from_list,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(Tag), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "Tags"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#cfn-transfer-server-tags"""

    
    @property
    def rv_ServerId(self) -> GetAtt:
        """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#aws-resource-transfer-server-return-values"""
        return GetAtt(resource=self, attr_name="ServerId")
    
    @property
    def rv_Arn(self) -> GetAtt:
        """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-server.html#aws-resource-transfer-server-return-values"""
        return GetAtt(resource=self, attr_name="Arn")
    

@attr.s
class User(Resource):
    """
    AWS Object Type = "AWS::Transfer::User"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html

    Property Document:
    
    - ``rp_Role``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-role
    - ``rp_ServerId``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-serverid
    - ``rp_UserName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-username
    - ``p_HomeDirectory``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-homedirectory
    - ``p_HomeDirectoryMappings``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-homedirectorymappings
    - ``p_HomeDirectoryType``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-homedirectorytype
    - ``p_Policy``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-policy
    - ``p_PosixProfile``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-posixprofile
    - ``p_SshPublicKeys``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-sshpublickeys
    - ``p_Tags``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-tags
    """
    AWS_OBJECT_TYPE = "AWS::Transfer::User"

    
    rp_Role: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "Role"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-role"""
    rp_ServerId: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "ServerId"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-serverid"""
    rp_UserName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "UserName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-username"""
    p_HomeDirectory: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "HomeDirectory"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-homedirectory"""
    p_HomeDirectoryMappings: typing.List[typing.Union['UserHomeDirectoryMapEntry', dict]] = attr.ib(
        default=None,
        converter=UserHomeDirectoryMapEntry.from_list,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(UserHomeDirectoryMapEntry), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "HomeDirectoryMappings"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-homedirectorymappings"""
    p_HomeDirectoryType: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "HomeDirectoryType"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-homedirectorytype"""
    p_Policy: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "Policy"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-policy"""
    p_PosixProfile: typing.Union['UserPosixProfile', dict] = attr.ib(
        default=None,
        converter=UserPosixProfile.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(UserPosixProfile)),
        metadata={AttrMeta.PROPERTY_NAME: "PosixProfile"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-posixprofile"""
    p_SshPublicKeys: typing.List[typing.Union['UserSshPublicKey', dict]] = attr.ib(
        default=None,
        converter=UserSshPublicKey.from_list,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(UserSshPublicKey), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "SshPublicKeys"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-sshpublickeys"""
    p_Tags: typing.List[typing.Union[Tag, dict]] = attr.ib(
        default=None,
        converter=Tag.from_list,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(Tag), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "Tags"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#cfn-transfer-user-tags"""

    
    @property
    def rv_ServerId(self) -> GetAtt:
        """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#aws-resource-transfer-user-return-values"""
        return GetAtt(resource=self, attr_name="ServerId")
    
    @property
    def rv_UserName(self) -> GetAtt:
        """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#aws-resource-transfer-user-return-values"""
        return GetAtt(resource=self, attr_name="UserName")
    
    @property
    def rv_Arn(self) -> GetAtt:
        """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-transfer-user.html#aws-resource-transfer-user-return-values"""
        return GetAtt(resource=self, attr_name="Arn")
    



# -*- coding: utf-8 -*-

class _ServicePrincipalMixin:
    @classmethod
    def a4b(cls): return cls._build("a4b.amazonaws.com") # pragma: no cover

    @classmethod
    def acm_pca(cls): return cls._build("acm-pca.amazonaws.com") # pragma: no cover

    @classmethod
    def acm(cls): return cls._build("acm.amazonaws.com") # pragma: no cover

    @classmethod
    def alexa_app(cls): return cls._build("alexa-app.amazonaws.com") # pragma: no cover

    @classmethod
    def alexa_connectedh(cls): return cls._build("alexa-connectedh.amazonaws.com") # pragma: no cover

    @classmethod
    def amazonmq(cls): return cls._build("amazonmq.amazonaws.com") # pragma: no cover

    @classmethod
    def apigateway(cls): return cls._build("apigateway.amazonaws.com") # pragma: no cover

    @classmethod
    def appflow(cls): return cls._build("appflow.amazonaws.com") # pragma: no cover

    @classmethod
    def application_autoscaling(cls): return cls._build("application-autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def appstream_application_autoscaling(cls): return cls._build("appstream.application-autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def appsync(cls): return cls._build("appsync.amazonaws.com") # pragma: no cover

    @classmethod
    def athena(cls): return cls._build("athena.amazonaws.com") # pragma: no cover

    @classmethod
    def autoscaling(cls): return cls._build("autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def aws_artifact_account_sync(cls): return cls._build("aws-artifact-account-sync.amazonaws.com") # pragma: no cover

    @classmethod
    def backup(cls): return cls._build("backup.amazonaws.com") # pragma: no cover

    @classmethod
    def batch(cls): return cls._build("batch.amazonaws.com") # pragma: no cover

    @classmethod
    def billingconsole(cls): return cls._build("billingconsole.amazonaws.com") # pragma: no cover

    @classmethod
    def budgets(cls): return cls._build("budgets.amazonaws.com") # pragma: no cover

    @classmethod
    def ce(cls): return cls._build("ce.amazonaws.com") # pragma: no cover

    @classmethod
    def channels_lex(cls): return cls._build("channels.lex.amazonaws.com") # pragma: no cover

    @classmethod
    def chime(cls): return cls._build("chime.amazonaws.com") # pragma: no cover

    @classmethod
    def cloud9(cls): return cls._build("cloud9.amazonaws.com") # pragma: no cover

    @classmethod
    def clouddirectory(cls): return cls._build("clouddirectory.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudformation(cls): return cls._build("cloudformation.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudfront(cls): return cls._build("cloudfront.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudhsm(cls): return cls._build("cloudhsm.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudsearch(cls): return cls._build("cloudsearch.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudtrail(cls): return cls._build("cloudtrail.amazonaws.com") # pragma: no cover

    @classmethod
    def cloudwatch_crossaccount(cls): return cls._build("cloudwatch-crossaccount.amazonaws.com") # pragma: no cover

    @classmethod
    def codebuild(cls): return cls._build("codebuild.amazonaws.com") # pragma: no cover

    @classmethod
    def codecommit(cls): return cls._build("codecommit.amazonaws.com") # pragma: no cover

    @classmethod
    def codedeploy(cls): return cls._build("codedeploy.${AWS::Region}.amazonaws.com") # pragma: no cover

    @classmethod
    def codedeploy(cls): return cls._build("codedeploy.amazonaws.com") # pragma: no cover

    @classmethod
    def codepipeline(cls): return cls._build("codepipeline.amazonaws.com") # pragma: no cover

    @classmethod
    def codestar(cls): return cls._build("codestar.amazonaws.com") # pragma: no cover

    @classmethod
    def cognito_identity(cls): return cls._build("cognito-identity.amazonaws.com") # pragma: no cover

    @classmethod
    def cognito_idp(cls): return cls._build("cognito-idp.amazonaws.com") # pragma: no cover

    @classmethod
    def cognito_sync(cls): return cls._build("cognito-sync.amazonaws.com") # pragma: no cover

    @classmethod
    def config_conforms(cls): return cls._build("config-conforms.amazonaws.com") # pragma: no cover

    @classmethod
    def config_multiaccountsetup(cls): return cls._build("config-multiaccountsetup.amazonaws.com") # pragma: no cover

    @classmethod
    def config(cls): return cls._build("config.amazonaws.com") # pragma: no cover

    @classmethod
    def connect(cls): return cls._build("connect.amazonaws.com") # pragma: no cover

    @classmethod
    def continuousexport_discovery(cls): return cls._build("continuousexport.discovery.amazonaws.com") # pragma: no cover

    @classmethod
    def custom_resource_application_autoscaling(cls): return cls._build("custom-resource.application-autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def databrew(cls): return cls._build("databrew.amazonaws.com") # pragma: no cover

    @classmethod
    def datapipeline(cls): return cls._build("datapipeline.amazonaws.com") # pragma: no cover

    @classmethod
    def dax(cls): return cls._build("dax.amazonaws.com") # pragma: no cover

    @classmethod
    def deeplens(cls): return cls._build("deeplens.amazonaws.com") # pragma: no cover

    @classmethod
    def delivery_logs(cls): return cls._build("delivery.logs.amazonaws.com") # pragma: no cover

    @classmethod
    def diode(cls): return cls._build("diode.amazonaws.com") # pragma: no cover

    @classmethod
    def directconnect(cls): return cls._build("directconnect.amazonaws.com") # pragma: no cover

    @classmethod
    def discovery(cls): return cls._build("discovery.amazonaws.com") # pragma: no cover

    @classmethod
    def dlm(cls): return cls._build("dlm.amazonaws.com") # pragma: no cover

    @classmethod
    def dms(cls): return cls._build("dms.amazonaws.com") # pragma: no cover

    @classmethod
    def ds(cls): return cls._build("ds.amazonaws.com") # pragma: no cover

    @classmethod
    def dynamodb(cls): return cls._build("dynamodb.amazonaws.com") # pragma: no cover

    @classmethod
    def dynamodb_application_autoscaling(cls): return cls._build("dynamodb.application-autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def ec(cls): return cls._build("ec.amazonaws.com") # pragma: no cover

    @classmethod
    def ec2(cls): return cls._build("ec2.amazonaws.com") # pragma: no cover

    @classmethod
    def ec2_application_autoscaling(cls): return cls._build("ec2.application-autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def ec2fleet(cls): return cls._build("ec2fleet.amazonaws.com") # pragma: no cover

    @classmethod
    def ec2scheduled(cls): return cls._build("ec2scheduled.amazonaws.com") # pragma: no cover

    @classmethod
    def ecr(cls): return cls._build("ecr.amazonaws.com") # pragma: no cover

    @classmethod
    def ecs_tasks(cls): return cls._build("ecs-tasks.amazonaws.com") # pragma: no cover

    @classmethod
    def ecs(cls): return cls._build("ecs.amazonaws.com") # pragma: no cover

    @classmethod
    def ecs_application_autoscaling(cls): return cls._build("ecs.application-autoscaling.amazonaws.com") # pragma: no cover

    @classmethod
    def edgelambda(cls): return cls._build("edgelambda.amazonaws.com") # pragma: no cover

    @classmethod
    def eks(cls): return cls._build("eks.amazonaws.com") # pragma: no cover

    @classmethod
    def elasticache(cls): return cls._build("elasticache.amazonaws.com") # pragma: no cover

    @classmethod
    def elasticbeanstalk(cls): return cls._build("elasticbeanstalk.amazonaws.com") # pragma: no cover

    @classmethod
    def elasticfilesystem(cls): return cls._build("elasticfilesystem.amazonaws.com") # pragma: no cover

    @classmethod
    def elasticloadbalancing(cls): return cls._build("elasticloadbalancing.amazonaws.com") # pragma: no cover

    @classmethod
    def elasticmapreduce(cls): return cls._build("elasticmapreduce.amazonaws.com") # pragma: no cover

    @classmethod
    def elastictranscoder(cls): return cls._build("elastictranscoder.amazonaws.com") # pragma: no cover

    @classmethod
    def email_cognito_idp(cls): return cls._build("email.cognito-idp.amazonaws.com") # pragma: no cover

    @classmethod
    def es(cls): return cls._build("es.amazonaws.com") # pragma: no cover

    @classmethod
    def events(cls): return cls._build("events.amazonaws.com") # pragma: no cover

    @classmethod
    def firehose(cls): return cls._build("firehose.amazonaws.com") # pragma: no cover

    @classmethod
    def fms(cls): return cls._build("fms.amazonaws.com") # pragma: no cover

    @classmethod
    def freertos(cls): return cls._build("freertos.amazonaws.com") # pragma: no cover

    @classmethod
    def fsx(cls): return cls._build("fsx.amazonaws.com") # pragma: no cover

    @classmethod
    def gamelift(cls): return cls._build("gamelift.amazonaws.com") # pragma: no cover

    @classmethod
    def glacier(cls): return cls._build("glacier.amazonaws.com") # pragma: no cover

    @classmethod
    def globalaccelerator(cls): return cls._build("globalaccelerator.amazonaws.com") # pragma: no cover

    @classmethod
    def glue(cls): return cls._build("glue.amazonaws.com") # pragma: no cover

    @classmethod
    def greengrass(cls): return cls._build("greengrass.amazonaws.com") # pragma: no cover

    @classmethod
    def guardduty(cls): return cls._build("guardduty.amazonaws.com") # pragma: no cover

    @classmethod
    def health(cls): return cls._build("health.amazonaws.com") # pragma: no cover

    @classmethod
    def iam(cls): return cls._build("iam.amazonaws.com") # pragma: no cover

    @classmethod
    def importexport(cls): return cls._build("importexport.amazonaws.com") # pragma: no cover

    @classmethod
    def inspector(cls): return cls._build("inspector.amazonaws.com") # pragma: no cover

    @classmethod
    def iot(cls): return cls._build("iot.amazonaws.com") # pragma: no cover

    @classmethod
    def iotanalytics(cls): return cls._build("iotanalytics.amazonaws.com") # pragma: no cover

    @classmethod
    def iotevents(cls): return cls._build("iotevents.amazonaws.com") # pragma: no cover

    @classmethod
    def iotsitewise(cls): return cls._build("iotsitewise.amazonaws.com") # pragma: no cover

    @classmethod
    def iotthingsgraph(cls): return cls._build("iotthingsgraph.amazonaws.com") # pragma: no cover

    @classmethod
    def jellyfish(cls): return cls._build("jellyfish.amazonaws.com") # pragma: no cover

    @classmethod
    def kafka(cls): return cls._build("kafka.amazonaws.com") # pragma: no cover

    @classmethod
    def kinesis(cls): return cls._build("kinesis.amazonaws.com") # pragma: no cover

    @classmethod
    def kinesisanalytics(cls): return cls._build("kinesisanalytics.amazonaws.com") # pragma: no cover

    @classmethod
    def kms(cls): return cls._build("kms.amazonaws.com") # pragma: no cover

    @classmethod
    def lakeformation(cls): return cls._build("lakeformation.amazonaws.com") # pragma: no cover

    @classmethod
    def awslambda(cls): return cls._build("lambda.amazonaws.com") # pragma: no cover

    @classmethod
    def lex(cls): return cls._build("lex.amazonaws.com") # pragma: no cover

    @classmethod
    def license_manager(cls): return cls._build("license-manager.amazonaws.com") # pragma: no cover

    @classmethod
    def lightsail(cls): return cls._build("lightsail.amazonaws.com") # pragma: no cover

    @classmethod
    def logger_cloudfront(cls): return cls._build("logger.cloudfront.amazonaws.com") # pragma: no cover

    @classmethod
    def logs(cls): return cls._build("logs.amazonaws.com") # pragma: no cover

    @classmethod
    def machinelearning(cls): return cls._build("machinelearning.amazonaws.com") # pragma: no cover

    @classmethod
    def macie(cls): return cls._build("macie.amazonaws.com") # pragma: no cover

    @classmethod
    def managedservices(cls): return cls._build("managedservices.amazonaws.com") # pragma: no cover

    @classmethod
    def mediaconnect(cls): return cls._build("mediaconnect.amazonaws.com") # pragma: no cover

    @classmethod
    def mediaconvert(cls): return cls._build("mediaconvert.amazonaws.com") # pragma: no cover

    @classmethod
    def mediapackage(cls): return cls._build("mediapackage.amazonaws.com") # pragma: no cover

    @classmethod
    def mediastore(cls): return cls._build("mediastore.amazonaws.com") # pragma: no cover

    @classmethod
    def mediatailor(cls): return cls._build("mediatailor.amazonaws.com") # pragma: no cover

    @classmethod
    def member_org_stacksets_cloudformation(cls): return cls._build("member.org.stacksets.cloudformation.amazonaws.com") # pragma: no cover

    @classmethod
    def metering_marketplace(cls): return cls._build("metering-marketplace.amazonaws.com") # pragma: no cover

    @classmethod
    def migrationhub(cls): return cls._build("migrationhub.amazonaws.com") # pragma: no cover

    @classmethod
    def mobilehub(cls): return cls._build("mobilehub.amazonaws.com") # pragma: no cover

    @classmethod
    def monitoring(cls): return cls._build("monitoring.amazonaws.com") # pragma: no cover

    @classmethod
    def monitoring_rds(cls): return cls._build("monitoring.rds.amazonaws.com") # pragma: no cover

    @classmethod
    def ops_apigateway(cls): return cls._build("ops.apigateway.amazonaws.com") # pragma: no cover

    @classmethod
    def opsworks_cm(cls): return cls._build("opsworks-cm.amazonaws.com") # pragma: no cover

    @classmethod
    def opsworks(cls): return cls._build("opsworks.amazonaws.com") # pragma: no cover

    @classmethod
    def organizations(cls): return cls._build("organizations.amazonaws.com") # pragma: no cover

    @classmethod
    def pinpoint(cls): return cls._build("pinpoint.amazonaws.com") # pragma: no cover

    @classmethod
    def polly(cls): return cls._build("polly.amazonaws.com") # pragma: no cover

    @classmethod
    def qldb(cls): return cls._build("qldb.amazonaws.com") # pragma: no cover

    @classmethod
    def quicksight(cls): return cls._build("quicksight.amazonaws.com") # pragma: no cover

    @classmethod
    def ram(cls): return cls._build("ram.amazonaws.com") # pragma: no cover

    @classmethod
    def rds(cls): return cls._build("rds.amazonaws.com") # pragma: no cover

    @classmethod
    def redshift(cls): return cls._build("redshift.amazonaws.com") # pragma: no cover

    @classmethod
    def rekognition(cls): return cls._build("rekognition.amazonaws.com") # pragma: no cover

    @classmethod
    def replication_dynamodb(cls): return cls._build("replication.dynamodb.amazonaws.com") # pragma: no cover

    @classmethod
    def replicator_lambda(cls): return cls._build("replicator.lambda.amazonaws.com") # pragma: no cover

    @classmethod
    def resource_groups(cls): return cls._build("resource-groups.amazonaws.com") # pragma: no cover

    @classmethod
    def robomaker(cls): return cls._build("robomaker.amazonaws.com") # pragma: no cover

    @classmethod
    def route53(cls): return cls._build("route53.amazonaws.com") # pragma: no cover

    @classmethod
    def route53domains(cls): return cls._build("route53domains.amazonaws.com") # pragma: no cover

    @classmethod
    def route53resolver(cls): return cls._build("route53resolver.amazonaws.com") # pragma: no cover

    @classmethod
    def s3(cls): return cls._build("s3.amazonaws.com") # pragma: no cover

    @classmethod
    def sagemaker(cls): return cls._build("sagemaker.amazonaws.com") # pragma: no cover

    @classmethod
    def secretsmanager(cls): return cls._build("secretsmanager.amazonaws.com") # pragma: no cover

    @classmethod
    def securityhub(cls): return cls._build("securityhub.amazonaws.com") # pragma: no cover

    @classmethod
    def serverlessrepo(cls): return cls._build("serverlessrepo.amazonaws.com") # pragma: no cover

    @classmethod
    def servicecatalog(cls): return cls._build("servicecatalog.amazonaws.com") # pragma: no cover

    @classmethod
    def servicediscovery(cls): return cls._build("servicediscovery.amazonaws.com") # pragma: no cover

    @classmethod
    def ses(cls): return cls._build("ses.amazonaws.com") # pragma: no cover

    @classmethod
    def shield(cls): return cls._build("shield.amazonaws.com") # pragma: no cover

    @classmethod
    def signer(cls): return cls._build("signer.amazonaws.com") # pragma: no cover

    @classmethod
    def signin(cls): return cls._build("signin.amazonaws.com") # pragma: no cover

    @classmethod
    def sms(cls): return cls._build("sms.amazonaws.com") # pragma: no cover

    @classmethod
    def sns(cls): return cls._build("sns.amazonaws.com") # pragma: no cover

    @classmethod
    def spotfleet(cls): return cls._build("spotfleet.amazonaws.com") # pragma: no cover

    @classmethod
    def sqs(cls): return cls._build("sqs.amazonaws.com") # pragma: no cover

    @classmethod
    def ssm(cls): return cls._build("ssm.amazonaws.com") # pragma: no cover

    @classmethod
    def sso(cls): return cls._build("sso.amazonaws.com") # pragma: no cover

    @classmethod
    def states(cls): return cls._build("states.amazonaws.com") # pragma: no cover

    @classmethod
    def storagegateway(cls): return cls._build("storagegateway.amazonaws.com") # pragma: no cover

    @classmethod
    def sts(cls): return cls._build("sts.amazonaws.com") # pragma: no cover

    @classmethod
    def support(cls): return cls._build("support.amazonaws.com") # pragma: no cover

    @classmethod
    def swf(cls): return cls._build("swf.amazonaws.com") # pragma: no cover

    @classmethod
    def tagging(cls): return cls._build("tagging.amazonaws.com") # pragma: no cover

    @classmethod
    def tagpolicies_tag(cls): return cls._build("tagpolicies.tag.amazonaws.com") # pragma: no cover

    @classmethod
    def transcribe(cls): return cls._build("transcribe.amazonaws.com") # pragma: no cover

    @classmethod
    def transfer(cls): return cls._build("transfer.amazonaws.com") # pragma: no cover

    @classmethod
    def translate(cls): return cls._build("translate.amazonaws.com") # pragma: no cover

    @classmethod
    def trustedadvisor(cls): return cls._build("trustedadvisor.amazonaws.com") # pragma: no cover

    @classmethod
    def tts(cls): return cls._build("tts.amazonaws.com") # pragma: no cover

    @classmethod
    def vmie(cls): return cls._build("vmie.amazonaws.com") # pragma: no cover

    @classmethod
    def waf_regional(cls): return cls._build("waf-regional.amazonaws.com") # pragma: no cover

    @classmethod
    def waf(cls): return cls._build("waf.amazonaws.com") # pragma: no cover

    @classmethod
    def workdocs(cls): return cls._build("workdocs.amazonaws.com") # pragma: no cover

    @classmethod
    def worklink(cls): return cls._build("worklink.amazonaws.com") # pragma: no cover

    @classmethod
    def workmail(cls): return cls._build("workmail.amazonaws.com") # pragma: no cover

    @classmethod
    def workspaces(cls): return cls._build("workspaces.amazonaws.com") # pragma: no cover

    @classmethod
    def xray(cls): return cls._build("xray.amazonaws.com") # pragma: no cover



# -*- coding: utf-8 -*-

"""
This module
"""

import attr
import typing

from ..core.model import (
    Property, Resource, Tag, GetAtt, TypeHint, TypeCheck,
)
from ..core.constant import AttrMeta

#--- Property declaration ---

@attr.s
class GlobalTablePointInTimeRecoverySpecification(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.PointInTimeRecoverySpecification"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-pointintimerecoveryspecification.html

    Property Document:
    
    - ``p_PointInTimeRecoveryEnabled``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-pointintimerecoveryspecification.html#cfn-dynamodb-globaltable-pointintimerecoveryspecification-pointintimerecoveryenabled
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.PointInTimeRecoverySpecification"
    
    p_PointInTimeRecoveryEnabled: bool = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(bool)),
        metadata={AttrMeta.PROPERTY_NAME: "PointInTimeRecoveryEnabled"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-pointintimerecoveryspecification.html#cfn-dynamodb-globaltable-pointintimerecoveryspecification-pointintimerecoveryenabled"""

@attr.s
class TablePointInTimeRecoverySpecification(Property):
    """
    AWS Object Type = "AWS::DynamoDB::Table.PointInTimeRecoverySpecification"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-pointintimerecoveryspecification.html

    Property Document:
    
    - ``p_PointInTimeRecoveryEnabled``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-pointintimerecoveryspecification.html#cfn-dynamodb-table-pointintimerecoveryspecification-pointintimerecoveryenabled
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::Table.PointInTimeRecoverySpecification"
    
    p_PointInTimeRecoveryEnabled: bool = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(bool)),
        metadata={AttrMeta.PROPERTY_NAME: "PointInTimeRecoveryEnabled"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-pointintimerecoveryspecification.html#cfn-dynamodb-table-pointintimerecoveryspecification-pointintimerecoveryenabled"""

@attr.s
class TableKinesisStreamSpecification(Property):
    """
    AWS Object Type = "AWS::DynamoDB::Table.KinesisStreamSpecification"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-kinesisstreamspecification.html

    Property Document:
    
    - ``rp_StreamArn``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-kinesisstreamspecification.html#cfn-dynamodb-kinesisstreamspecification-streamarn
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::Table.KinesisStreamSpecification"
    
    rp_StreamArn: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "StreamArn"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-kinesisstreamspecification.html#cfn-dynamodb-kinesisstreamspecification-streamarn"""

@attr.s
class GlobalTableContributorInsightsSpecification(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.ContributorInsightsSpecification"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-contributorinsightsspecification.html

    Property Document:
    
    - ``rp_Enabled``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-contributorinsightsspecification.html#cfn-dynamodb-globaltable-contributorinsightsspecification-enabled
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.ContributorInsightsSpecification"
    
    rp_Enabled: bool = attr.ib(
        default=None,
        validator=attr.validators.instance_of(bool),
        metadata={AttrMeta.PROPERTY_NAME: "Enabled"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-contributorinsightsspecification.html#cfn-dynamodb-globaltable-contributorinsightsspecification-enabled"""

@attr.s
class TableAttributeDefinition(Property):
    """
    AWS Object Type = "AWS::DynamoDB::Table.AttributeDefinition"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-attributedef.html

    Property Document:
    
    - ``rp_AttributeName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-attributedef.html#cfn-dynamodb-attributedef-attributename
    - ``rp_AttributeType``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-attributedef.html#cfn-dynamodb-attributedef-attributename-attributetype
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::Table.AttributeDefinition"
    
    rp_AttributeName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "AttributeName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-attributedef.html#cfn-dynamodb-attributedef-attributename"""
    rp_AttributeType: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "AttributeType"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-attributedef.html#cfn-dynamodb-attributedef-attributename-attributetype"""

@attr.s
class TableContributorInsightsSpecification(Property):
    """
    AWS Object Type = "AWS::DynamoDB::Table.ContributorInsightsSpecification"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-contributorinsightsspecification.html

    Property Document:
    
    - ``rp_Enabled``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-contributorinsightsspecification.html#cfn-dynamodb-contributorinsightsspecification-enabled
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::Table.ContributorInsightsSpecification"
    
    rp_Enabled: bool = attr.ib(
        default=None,
        validator=attr.validators.instance_of(bool),
        metadata={AttrMeta.PROPERTY_NAME: "Enabled"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-contributorinsightsspecification.html#cfn-dynamodb-contributorinsightsspecification-enabled"""

@attr.s
class TableKeySchema(Property):
    """
    AWS Object Type = "AWS::DynamoDB::Table.KeySchema"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-keyschema.html

    Property Document:
    
    - ``rp_AttributeName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-keyschema.html#aws-properties-dynamodb-keyschema-attributename
    - ``rp_KeyType``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-keyschema.html#aws-properties-dynamodb-keyschema-keytype
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::Table.KeySchema"
    
    rp_AttributeName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "AttributeName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-keyschema.html#aws-properties-dynamodb-keyschema-attributename"""
    rp_KeyType: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "KeyType"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-keyschema.html#aws-properties-dynamodb-keyschema-keytype"""

@attr.s
class GlobalTableTargetTrackingScalingPolicyConfiguration(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.TargetTrackingScalingPolicyConfiguration"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-targettrackingscalingpolicyconfiguration.html

    Property Document:
    
    - ``rp_TargetValue``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-targettrackingscalingpolicyconfiguration.html#cfn-dynamodb-globaltable-targettrackingscalingpolicyconfiguration-targetvalue
    - ``p_DisableScaleIn``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-targettrackingscalingpolicyconfiguration.html#cfn-dynamodb-globaltable-targettrackingscalingpolicyconfiguration-disablescalein
    - ``p_ScaleInCooldown``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-targettrackingscalingpolicyconfiguration.html#cfn-dynamodb-globaltable-targettrackingscalingpolicyconfiguration-scaleincooldown
    - ``p_ScaleOutCooldown``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-targettrackingscalingpolicyconfiguration.html#cfn-dynamodb-globaltable-targettrackingscalingpolicyconfiguration-scaleoutcooldown
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.TargetTrackingScalingPolicyConfiguration"
    
    rp_TargetValue: float = attr.ib(
        default=None,
        validator=attr.validators.instance_of(float),
        metadata={AttrMeta.PROPERTY_NAME: "TargetValue"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-targettrackingscalingpolicyconfiguration.html#cfn-dynamodb-globaltable-targettrackingscalingpolicyconfiguration-targetvalue"""
    p_DisableScaleIn: bool = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(bool)),
        metadata={AttrMeta.PROPERTY_NAME: "DisableScaleIn"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-targettrackingscalingpolicyconfiguration.html#cfn-dynamodb-globaltable-targettrackingscalingpolicyconfiguration-disablescalein"""
    p_ScaleInCooldown: int = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(int)),
        metadata={AttrMeta.PROPERTY_NAME: "ScaleInCooldown"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-targettrackingscalingpolicyconfiguration.html#cfn-dynamodb-globaltable-targettrackingscalingpolicyconfiguration-scaleincooldown"""
    p_ScaleOutCooldown: int = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(int)),
        metadata={AttrMeta.PROPERTY_NAME: "ScaleOutCooldown"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-targettrackingscalingpolicyconfiguration.html#cfn-dynamodb-globaltable-targettrackingscalingpolicyconfiguration-scaleoutcooldown"""

@attr.s
class GlobalTableKeySchema(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.KeySchema"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-keyschema.html

    Property Document:
    
    - ``rp_AttributeName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-keyschema.html#cfn-dynamodb-globaltable-keyschema-attributename
    - ``rp_KeyType``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-keyschema.html#cfn-dynamodb-globaltable-keyschema-keytype
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.KeySchema"
    
    rp_AttributeName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "AttributeName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-keyschema.html#cfn-dynamodb-globaltable-keyschema-attributename"""
    rp_KeyType: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "KeyType"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-keyschema.html#cfn-dynamodb-globaltable-keyschema-keytype"""

@attr.s
class GlobalTableStreamSpecification(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.StreamSpecification"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-streamspecification.html

    Property Document:
    
    - ``rp_StreamViewType``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-streamspecification.html#cfn-dynamodb-globaltable-streamspecification-streamviewtype
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.StreamSpecification"
    
    rp_StreamViewType: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "StreamViewType"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-streamspecification.html#cfn-dynamodb-globaltable-streamspecification-streamviewtype"""

@attr.s
class GlobalTableProjection(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.Projection"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-projection.html

    Property Document:
    
    - ``p_NonKeyAttributes``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-projection.html#cfn-dynamodb-globaltable-projection-nonkeyattributes
    - ``p_ProjectionType``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-projection.html#cfn-dynamodb-globaltable-projection-projectiontype
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.Projection"
    
    p_NonKeyAttributes: typing.List[TypeHint.intrinsic_str] = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "NonKeyAttributes"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-projection.html#cfn-dynamodb-globaltable-projection-nonkeyattributes"""
    p_ProjectionType: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "ProjectionType"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-projection.html#cfn-dynamodb-globaltable-projection-projectiontype"""

@attr.s
class GlobalTableAttributeDefinition(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.AttributeDefinition"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-attributedefinition.html

    Property Document:
    
    - ``rp_AttributeName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-attributedefinition.html#cfn-dynamodb-globaltable-attributedefinition-attributename
    - ``rp_AttributeType``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-attributedefinition.html#cfn-dynamodb-globaltable-attributedefinition-attributetype
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.AttributeDefinition"
    
    rp_AttributeName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "AttributeName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-attributedefinition.html#cfn-dynamodb-globaltable-attributedefinition-attributename"""
    rp_AttributeType: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "AttributeType"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-attributedefinition.html#cfn-dynamodb-globaltable-attributedefinition-attributetype"""

@attr.s
class GlobalTableSSESpecification(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.SSESpecification"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-ssespecification.html

    Property Document:
    
    - ``rp_SSEEnabled``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-ssespecification.html#cfn-dynamodb-globaltable-ssespecification-sseenabled
    - ``p_SSEType``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-ssespecification.html#cfn-dynamodb-globaltable-ssespecification-ssetype
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.SSESpecification"
    
    rp_SSEEnabled: bool = attr.ib(
        default=None,
        validator=attr.validators.instance_of(bool),
        metadata={AttrMeta.PROPERTY_NAME: "SSEEnabled"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-ssespecification.html#cfn-dynamodb-globaltable-ssespecification-sseenabled"""
    p_SSEType: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "SSEType"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-ssespecification.html#cfn-dynamodb-globaltable-ssespecification-ssetype"""

@attr.s
class TableSSESpecification(Property):
    """
    AWS Object Type = "AWS::DynamoDB::Table.SSESpecification"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-ssespecification.html

    Property Document:
    
    - ``rp_SSEEnabled``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-ssespecification.html#cfn-dynamodb-table-ssespecification-sseenabled
    - ``p_KMSMasterKeyId``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-ssespecification.html#cfn-dynamodb-table-ssespecification-kmsmasterkeyid
    - ``p_SSEType``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-ssespecification.html#cfn-dynamodb-table-ssespecification-ssetype
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::Table.SSESpecification"
    
    rp_SSEEnabled: bool = attr.ib(
        default=None,
        validator=attr.validators.instance_of(bool),
        metadata={AttrMeta.PROPERTY_NAME: "SSEEnabled"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-ssespecification.html#cfn-dynamodb-table-ssespecification-sseenabled"""
    p_KMSMasterKeyId: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "KMSMasterKeyId"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-ssespecification.html#cfn-dynamodb-table-ssespecification-kmsmasterkeyid"""
    p_SSEType: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "SSEType"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-table-ssespecification.html#cfn-dynamodb-table-ssespecification-ssetype"""

@attr.s
class TableTimeToLiveSpecification(Property):
    """
    AWS Object Type = "AWS::DynamoDB::Table.TimeToLiveSpecification"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-timetolivespecification.html

    Property Document:
    
    - ``rp_AttributeName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-timetolivespecification.html#cfn-dynamodb-timetolivespecification-attributename
    - ``rp_Enabled``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-timetolivespecification.html#cfn-dynamodb-timetolivespecification-enabled
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::Table.TimeToLiveSpecification"
    
    rp_AttributeName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "AttributeName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-timetolivespecification.html#cfn-dynamodb-timetolivespecification-attributename"""
    rp_Enabled: bool = attr.ib(
        default=None,
        validator=attr.validators.instance_of(bool),
        metadata={AttrMeta.PROPERTY_NAME: "Enabled"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-timetolivespecification.html#cfn-dynamodb-timetolivespecification-enabled"""

@attr.s
class TableProvisionedThroughput(Property):
    """
    AWS Object Type = "AWS::DynamoDB::Table.ProvisionedThroughput"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-provisionedthroughput.html

    Property Document:
    
    - ``rp_ReadCapacityUnits``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-provisionedthroughput.html#cfn-dynamodb-provisionedthroughput-readcapacityunits
    - ``rp_WriteCapacityUnits``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-provisionedthroughput.html#cfn-dynamodb-provisionedthroughput-writecapacityunits
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::Table.ProvisionedThroughput"
    
    rp_ReadCapacityUnits: int = attr.ib(
        default=None,
        validator=attr.validators.instance_of(int),
        metadata={AttrMeta.PROPERTY_NAME: "ReadCapacityUnits"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-provisionedthroughput.html#cfn-dynamodb-provisionedthroughput-readcapacityunits"""
    rp_WriteCapacityUnits: int = attr.ib(
        default=None,
        validator=attr.validators.instance_of(int),
        metadata={AttrMeta.PROPERTY_NAME: "WriteCapacityUnits"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-provisionedthroughput.html#cfn-dynamodb-provisionedthroughput-writecapacityunits"""

@attr.s
class TableProjection(Property):
    """
    AWS Object Type = "AWS::DynamoDB::Table.Projection"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-projectionobject.html

    Property Document:
    
    - ``p_NonKeyAttributes``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-projectionobject.html#cfn-dynamodb-projectionobj-nonkeyatt
    - ``p_ProjectionType``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-projectionobject.html#cfn-dynamodb-projectionobj-projtype
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::Table.Projection"
    
    p_NonKeyAttributes: typing.List[TypeHint.intrinsic_str] = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "NonKeyAttributes"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-projectionobject.html#cfn-dynamodb-projectionobj-nonkeyatt"""
    p_ProjectionType: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "ProjectionType"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-projectionobject.html#cfn-dynamodb-projectionobj-projtype"""

@attr.s
class GlobalTableTimeToLiveSpecification(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.TimeToLiveSpecification"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-timetolivespecification.html

    Property Document:
    
    - ``rp_Enabled``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-timetolivespecification.html#cfn-dynamodb-globaltable-timetolivespecification-enabled
    - ``p_AttributeName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-timetolivespecification.html#cfn-dynamodb-globaltable-timetolivespecification-attributename
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.TimeToLiveSpecification"
    
    rp_Enabled: bool = attr.ib(
        default=None,
        validator=attr.validators.instance_of(bool),
        metadata={AttrMeta.PROPERTY_NAME: "Enabled"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-timetolivespecification.html#cfn-dynamodb-globaltable-timetolivespecification-enabled"""
    p_AttributeName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "AttributeName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-timetolivespecification.html#cfn-dynamodb-globaltable-timetolivespecification-attributename"""

@attr.s
class GlobalTableReplicaSSESpecification(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.ReplicaSSESpecification"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicassespecification.html

    Property Document:
    
    - ``rp_KMSMasterKeyId``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicassespecification.html#cfn-dynamodb-globaltable-replicassespecification-kmsmasterkeyid
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.ReplicaSSESpecification"
    
    rp_KMSMasterKeyId: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "KMSMasterKeyId"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicassespecification.html#cfn-dynamodb-globaltable-replicassespecification-kmsmasterkeyid"""

@attr.s
class TableStreamSpecification(Property):
    """
    AWS Object Type = "AWS::DynamoDB::Table.StreamSpecification"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-streamspecification.html

    Property Document:
    
    - ``rp_StreamViewType``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-streamspecification.html#cfn-dynamodb-streamspecification-streamviewtype
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::Table.StreamSpecification"
    
    rp_StreamViewType: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "StreamViewType"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-streamspecification.html#cfn-dynamodb-streamspecification-streamviewtype"""

@attr.s
class TableLocalSecondaryIndex(Property):
    """
    AWS Object Type = "AWS::DynamoDB::Table.LocalSecondaryIndex"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-lsi.html

    Property Document:
    
    - ``rp_IndexName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-lsi.html#cfn-dynamodb-lsi-indexname
    - ``rp_KeySchema``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-lsi.html#cfn-dynamodb-lsi-keyschema
    - ``rp_Projection``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-lsi.html#cfn-dynamodb-lsi-projection
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::Table.LocalSecondaryIndex"
    
    rp_IndexName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "IndexName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-lsi.html#cfn-dynamodb-lsi-indexname"""
    rp_KeySchema: typing.List[typing.Union['TableKeySchema', dict]] = attr.ib(
        default=None,
        converter=TableKeySchema.from_list,
        validator=attr.validators.deep_iterable(member_validator=attr.validators.instance_of(TableKeySchema), iterable_validator=attr.validators.instance_of(list)),
        metadata={AttrMeta.PROPERTY_NAME: "KeySchema"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-lsi.html#cfn-dynamodb-lsi-keyschema"""
    rp_Projection: typing.Union['TableProjection', dict] = attr.ib(
        default=None,
        converter=TableProjection.from_dict,
        validator=attr.validators.instance_of(TableProjection),
        metadata={AttrMeta.PROPERTY_NAME: "Projection"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-lsi.html#cfn-dynamodb-lsi-projection"""

@attr.s
class GlobalTableCapacityAutoScalingSettings(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.CapacityAutoScalingSettings"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-capacityautoscalingsettings.html

    Property Document:
    
    - ``rp_MaxCapacity``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-capacityautoscalingsettings.html#cfn-dynamodb-globaltable-capacityautoscalingsettings-maxcapacity
    - ``rp_MinCapacity``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-capacityautoscalingsettings.html#cfn-dynamodb-globaltable-capacityautoscalingsettings-mincapacity
    - ``rp_TargetTrackingScalingPolicyConfiguration``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-capacityautoscalingsettings.html#cfn-dynamodb-globaltable-capacityautoscalingsettings-targettrackingscalingpolicyconfiguration
    - ``p_SeedCapacity``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-capacityautoscalingsettings.html#cfn-dynamodb-globaltable-capacityautoscalingsettings-seedcapacity
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.CapacityAutoScalingSettings"
    
    rp_MaxCapacity: int = attr.ib(
        default=None,
        validator=attr.validators.instance_of(int),
        metadata={AttrMeta.PROPERTY_NAME: "MaxCapacity"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-capacityautoscalingsettings.html#cfn-dynamodb-globaltable-capacityautoscalingsettings-maxcapacity"""
    rp_MinCapacity: int = attr.ib(
        default=None,
        validator=attr.validators.instance_of(int),
        metadata={AttrMeta.PROPERTY_NAME: "MinCapacity"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-capacityautoscalingsettings.html#cfn-dynamodb-globaltable-capacityautoscalingsettings-mincapacity"""
    rp_TargetTrackingScalingPolicyConfiguration: typing.Union['GlobalTableTargetTrackingScalingPolicyConfiguration', dict] = attr.ib(
        default=None,
        converter=GlobalTableTargetTrackingScalingPolicyConfiguration.from_dict,
        validator=attr.validators.instance_of(GlobalTableTargetTrackingScalingPolicyConfiguration),
        metadata={AttrMeta.PROPERTY_NAME: "TargetTrackingScalingPolicyConfiguration"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-capacityautoscalingsettings.html#cfn-dynamodb-globaltable-capacityautoscalingsettings-targettrackingscalingpolicyconfiguration"""
    p_SeedCapacity: int = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(int)),
        metadata={AttrMeta.PROPERTY_NAME: "SeedCapacity"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-capacityautoscalingsettings.html#cfn-dynamodb-globaltable-capacityautoscalingsettings-seedcapacity"""

@attr.s
class GlobalTableReadProvisionedThroughputSettings(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.ReadProvisionedThroughputSettings"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-readprovisionedthroughputsettings.html

    Property Document:
    
    - ``p_ReadCapacityAutoScalingSettings``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-readprovisionedthroughputsettings.html#cfn-dynamodb-globaltable-readprovisionedthroughputsettings-readcapacityautoscalingsettings
    - ``p_ReadCapacityUnits``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-readprovisionedthroughputsettings.html#cfn-dynamodb-globaltable-readprovisionedthroughputsettings-readcapacityunits
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.ReadProvisionedThroughputSettings"
    
    p_ReadCapacityAutoScalingSettings: typing.Union['GlobalTableCapacityAutoScalingSettings', dict] = attr.ib(
        default=None,
        converter=GlobalTableCapacityAutoScalingSettings.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(GlobalTableCapacityAutoScalingSettings)),
        metadata={AttrMeta.PROPERTY_NAME: "ReadCapacityAutoScalingSettings"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-readprovisionedthroughputsettings.html#cfn-dynamodb-globaltable-readprovisionedthroughputsettings-readcapacityautoscalingsettings"""
    p_ReadCapacityUnits: int = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(int)),
        metadata={AttrMeta.PROPERTY_NAME: "ReadCapacityUnits"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-readprovisionedthroughputsettings.html#cfn-dynamodb-globaltable-readprovisionedthroughputsettings-readcapacityunits"""

@attr.s
class GlobalTableLocalSecondaryIndex(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.LocalSecondaryIndex"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-localsecondaryindex.html

    Property Document:
    
    - ``rp_IndexName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-localsecondaryindex.html#cfn-dynamodb-globaltable-localsecondaryindex-indexname
    - ``rp_KeySchema``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-localsecondaryindex.html#cfn-dynamodb-globaltable-localsecondaryindex-keyschema
    - ``rp_Projection``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-localsecondaryindex.html#cfn-dynamodb-globaltable-localsecondaryindex-projection
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.LocalSecondaryIndex"
    
    rp_IndexName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "IndexName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-localsecondaryindex.html#cfn-dynamodb-globaltable-localsecondaryindex-indexname"""
    rp_KeySchema: typing.List[typing.Union['GlobalTableKeySchema', dict]] = attr.ib(
        default=None,
        converter=GlobalTableKeySchema.from_list,
        validator=attr.validators.deep_iterable(member_validator=attr.validators.instance_of(GlobalTableKeySchema), iterable_validator=attr.validators.instance_of(list)),
        metadata={AttrMeta.PROPERTY_NAME: "KeySchema"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-localsecondaryindex.html#cfn-dynamodb-globaltable-localsecondaryindex-keyschema"""
    rp_Projection: typing.Union['GlobalTableProjection', dict] = attr.ib(
        default=None,
        converter=GlobalTableProjection.from_dict,
        validator=attr.validators.instance_of(GlobalTableProjection),
        metadata={AttrMeta.PROPERTY_NAME: "Projection"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-localsecondaryindex.html#cfn-dynamodb-globaltable-localsecondaryindex-projection"""

@attr.s
class TableGlobalSecondaryIndex(Property):
    """
    AWS Object Type = "AWS::DynamoDB::Table.GlobalSecondaryIndex"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html

    Property Document:
    
    - ``rp_IndexName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html#cfn-dynamodb-gsi-indexname
    - ``rp_KeySchema``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html#cfn-dynamodb-gsi-keyschema
    - ``rp_Projection``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html#cfn-dynamodb-gsi-projection
    - ``p_ContributorInsightsSpecification``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html#cfn-dynamodb-contributorinsightsspecification-enabled
    - ``p_ProvisionedThroughput``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html#cfn-dynamodb-gsi-provisionedthroughput
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::Table.GlobalSecondaryIndex"
    
    rp_IndexName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "IndexName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html#cfn-dynamodb-gsi-indexname"""
    rp_KeySchema: typing.List[typing.Union['TableKeySchema', dict]] = attr.ib(
        default=None,
        converter=TableKeySchema.from_list,
        validator=attr.validators.deep_iterable(member_validator=attr.validators.instance_of(TableKeySchema), iterable_validator=attr.validators.instance_of(list)),
        metadata={AttrMeta.PROPERTY_NAME: "KeySchema"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html#cfn-dynamodb-gsi-keyschema"""
    rp_Projection: typing.Union['TableProjection', dict] = attr.ib(
        default=None,
        converter=TableProjection.from_dict,
        validator=attr.validators.instance_of(TableProjection),
        metadata={AttrMeta.PROPERTY_NAME: "Projection"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html#cfn-dynamodb-gsi-projection"""
    p_ContributorInsightsSpecification: typing.Union['TableContributorInsightsSpecification', dict] = attr.ib(
        default=None,
        converter=TableContributorInsightsSpecification.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(TableContributorInsightsSpecification)),
        metadata={AttrMeta.PROPERTY_NAME: "ContributorInsightsSpecification"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html#cfn-dynamodb-contributorinsightsspecification-enabled"""
    p_ProvisionedThroughput: typing.Union['TableProvisionedThroughput', dict] = attr.ib(
        default=None,
        converter=TableProvisionedThroughput.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(TableProvisionedThroughput)),
        metadata={AttrMeta.PROPERTY_NAME: "ProvisionedThroughput"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-gsi.html#cfn-dynamodb-gsi-provisionedthroughput"""

@attr.s
class GlobalTableReplicaGlobalSecondaryIndexSpecification(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.ReplicaGlobalSecondaryIndexSpecification"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaglobalsecondaryindexspecification.html

    Property Document:
    
    - ``rp_IndexName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaglobalsecondaryindexspecification.html#cfn-dynamodb-globaltable-replicaglobalsecondaryindexspecification-indexname
    - ``p_ContributorInsightsSpecification``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaglobalsecondaryindexspecification.html#cfn-dynamodb-globaltable-replicaglobalsecondaryindexspecification-contributorinsightsspecification
    - ``p_ReadProvisionedThroughputSettings``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaglobalsecondaryindexspecification.html#cfn-dynamodb-globaltable-replicaglobalsecondaryindexspecification-readprovisionedthroughputsettings
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.ReplicaGlobalSecondaryIndexSpecification"
    
    rp_IndexName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "IndexName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaglobalsecondaryindexspecification.html#cfn-dynamodb-globaltable-replicaglobalsecondaryindexspecification-indexname"""
    p_ContributorInsightsSpecification: typing.Union['GlobalTableContributorInsightsSpecification', dict] = attr.ib(
        default=None,
        converter=GlobalTableContributorInsightsSpecification.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(GlobalTableContributorInsightsSpecification)),
        metadata={AttrMeta.PROPERTY_NAME: "ContributorInsightsSpecification"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaglobalsecondaryindexspecification.html#cfn-dynamodb-globaltable-replicaglobalsecondaryindexspecification-contributorinsightsspecification"""
    p_ReadProvisionedThroughputSettings: typing.Union['GlobalTableReadProvisionedThroughputSettings', dict] = attr.ib(
        default=None,
        converter=GlobalTableReadProvisionedThroughputSettings.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(GlobalTableReadProvisionedThroughputSettings)),
        metadata={AttrMeta.PROPERTY_NAME: "ReadProvisionedThroughputSettings"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaglobalsecondaryindexspecification.html#cfn-dynamodb-globaltable-replicaglobalsecondaryindexspecification-readprovisionedthroughputsettings"""

@attr.s
class GlobalTableWriteProvisionedThroughputSettings(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.WriteProvisionedThroughputSettings"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-writeprovisionedthroughputsettings.html

    Property Document:
    
    - ``p_WriteCapacityAutoScalingSettings``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-writeprovisionedthroughputsettings.html#cfn-dynamodb-globaltable-writeprovisionedthroughputsettings-writecapacityautoscalingsettings
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.WriteProvisionedThroughputSettings"
    
    p_WriteCapacityAutoScalingSettings: typing.Union['GlobalTableCapacityAutoScalingSettings', dict] = attr.ib(
        default=None,
        converter=GlobalTableCapacityAutoScalingSettings.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(GlobalTableCapacityAutoScalingSettings)),
        metadata={AttrMeta.PROPERTY_NAME: "WriteCapacityAutoScalingSettings"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-writeprovisionedthroughputsettings.html#cfn-dynamodb-globaltable-writeprovisionedthroughputsettings-writecapacityautoscalingsettings"""

@attr.s
class GlobalTableReplicaSpecification(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.ReplicaSpecification"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html

    Property Document:
    
    - ``rp_Region``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html#cfn-dynamodb-globaltable-replicaspecification-region
    - ``p_ContributorInsightsSpecification``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html#cfn-dynamodb-globaltable-replicaspecification-contributorinsightsspecification
    - ``p_GlobalSecondaryIndexes``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html#cfn-dynamodb-globaltable-replicaspecification-globalsecondaryindexes
    - ``p_PointInTimeRecoverySpecification``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html#cfn-dynamodb-globaltable-replicaspecification-pointintimerecoveryspecification
    - ``p_ReadProvisionedThroughputSettings``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html#cfn-dynamodb-globaltable-replicaspecification-readprovisionedthroughputsettings
    - ``p_SSESpecification``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html#cfn-dynamodb-globaltable-replicaspecification-ssespecification
    - ``p_Tags``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html#cfn-dynamodb-globaltable-replicaspecification-tags
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.ReplicaSpecification"
    
    rp_Region: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "Region"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html#cfn-dynamodb-globaltable-replicaspecification-region"""
    p_ContributorInsightsSpecification: typing.Union['GlobalTableContributorInsightsSpecification', dict] = attr.ib(
        default=None,
        converter=GlobalTableContributorInsightsSpecification.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(GlobalTableContributorInsightsSpecification)),
        metadata={AttrMeta.PROPERTY_NAME: "ContributorInsightsSpecification"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html#cfn-dynamodb-globaltable-replicaspecification-contributorinsightsspecification"""
    p_GlobalSecondaryIndexes: typing.List[typing.Union['GlobalTableReplicaGlobalSecondaryIndexSpecification', dict]] = attr.ib(
        default=None,
        converter=GlobalTableReplicaGlobalSecondaryIndexSpecification.from_list,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(GlobalTableReplicaGlobalSecondaryIndexSpecification), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "GlobalSecondaryIndexes"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html#cfn-dynamodb-globaltable-replicaspecification-globalsecondaryindexes"""
    p_PointInTimeRecoverySpecification: typing.Union['GlobalTablePointInTimeRecoverySpecification', dict] = attr.ib(
        default=None,
        converter=GlobalTablePointInTimeRecoverySpecification.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(GlobalTablePointInTimeRecoverySpecification)),
        metadata={AttrMeta.PROPERTY_NAME: "PointInTimeRecoverySpecification"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html#cfn-dynamodb-globaltable-replicaspecification-pointintimerecoveryspecification"""
    p_ReadProvisionedThroughputSettings: typing.Union['GlobalTableReadProvisionedThroughputSettings', dict] = attr.ib(
        default=None,
        converter=GlobalTableReadProvisionedThroughputSettings.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(GlobalTableReadProvisionedThroughputSettings)),
        metadata={AttrMeta.PROPERTY_NAME: "ReadProvisionedThroughputSettings"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html#cfn-dynamodb-globaltable-replicaspecification-readprovisionedthroughputsettings"""
    p_SSESpecification: typing.Union['GlobalTableReplicaSSESpecification', dict] = attr.ib(
        default=None,
        converter=GlobalTableReplicaSSESpecification.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(GlobalTableReplicaSSESpecification)),
        metadata={AttrMeta.PROPERTY_NAME: "SSESpecification"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html#cfn-dynamodb-globaltable-replicaspecification-ssespecification"""
    p_Tags: typing.List[typing.Union[Tag, dict]] = attr.ib(
        default=None,
        converter=Tag.from_list,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(Tag), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "Tags"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-replicaspecification.html#cfn-dynamodb-globaltable-replicaspecification-tags"""

@attr.s
class GlobalTableGlobalSecondaryIndex(Property):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable.GlobalSecondaryIndex"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-globalsecondaryindex.html

    Property Document:
    
    - ``rp_IndexName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-globalsecondaryindex.html#cfn-dynamodb-globaltable-globalsecondaryindex-indexname
    - ``rp_KeySchema``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-globalsecondaryindex.html#cfn-dynamodb-globaltable-globalsecondaryindex-keyschema
    - ``rp_Projection``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-globalsecondaryindex.html#cfn-dynamodb-globaltable-globalsecondaryindex-projection
    - ``p_WriteProvisionedThroughputSettings``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-globalsecondaryindex.html#cfn-dynamodb-globaltable-globalsecondaryindex-writeprovisionedthroughputsettings
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable.GlobalSecondaryIndex"
    
    rp_IndexName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.instance_of(TypeCheck.intrinsic_str_type),
        metadata={AttrMeta.PROPERTY_NAME: "IndexName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-globalsecondaryindex.html#cfn-dynamodb-globaltable-globalsecondaryindex-indexname"""
    rp_KeySchema: typing.List[typing.Union['GlobalTableKeySchema', dict]] = attr.ib(
        default=None,
        converter=GlobalTableKeySchema.from_list,
        validator=attr.validators.deep_iterable(member_validator=attr.validators.instance_of(GlobalTableKeySchema), iterable_validator=attr.validators.instance_of(list)),
        metadata={AttrMeta.PROPERTY_NAME: "KeySchema"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-globalsecondaryindex.html#cfn-dynamodb-globaltable-globalsecondaryindex-keyschema"""
    rp_Projection: typing.Union['GlobalTableProjection', dict] = attr.ib(
        default=None,
        converter=GlobalTableProjection.from_dict,
        validator=attr.validators.instance_of(GlobalTableProjection),
        metadata={AttrMeta.PROPERTY_NAME: "Projection"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-globalsecondaryindex.html#cfn-dynamodb-globaltable-globalsecondaryindex-projection"""
    p_WriteProvisionedThroughputSettings: typing.Union['GlobalTableWriteProvisionedThroughputSettings', dict] = attr.ib(
        default=None,
        converter=GlobalTableWriteProvisionedThroughputSettings.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(GlobalTableWriteProvisionedThroughputSettings)),
        metadata={AttrMeta.PROPERTY_NAME: "WriteProvisionedThroughputSettings"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-dynamodb-globaltable-globalsecondaryindex.html#cfn-dynamodb-globaltable-globalsecondaryindex-writeprovisionedthroughputsettings"""


#--- Resource declaration ---

@attr.s
class Table(Resource):
    """
    AWS Object Type = "AWS::DynamoDB::Table"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html

    Property Document:
    
    - ``rp_KeySchema``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-keyschema
    - ``p_AttributeDefinitions``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-attributedef
    - ``p_BillingMode``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-billingmode
    - ``p_ContributorInsightsSpecification``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-contributorinsightsspecification-enabled
    - ``p_GlobalSecondaryIndexes``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-gsi
    - ``p_KinesisStreamSpecification``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-kinesisstreamspecification
    - ``p_LocalSecondaryIndexes``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-lsi
    - ``p_PointInTimeRecoverySpecification``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-pointintimerecoveryspecification
    - ``p_ProvisionedThroughput``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-provisionedthroughput
    - ``p_SSESpecification``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-ssespecification
    - ``p_StreamSpecification``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-streamspecification
    - ``p_TableName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-tablename
    - ``p_TimeToLiveSpecification``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-timetolivespecification
    - ``p_Tags``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-tags
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::Table"

    
    rp_KeySchema: typing.List[typing.Union['TableKeySchema', dict]] = attr.ib(
        default=None,
        converter=TableKeySchema.from_list,
        validator=attr.validators.deep_iterable(member_validator=attr.validators.instance_of(TableKeySchema), iterable_validator=attr.validators.instance_of(list)),
        metadata={AttrMeta.PROPERTY_NAME: "KeySchema"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-keyschema"""
    p_AttributeDefinitions: typing.List[typing.Union['TableAttributeDefinition', dict]] = attr.ib(
        default=None,
        converter=TableAttributeDefinition.from_list,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(TableAttributeDefinition), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "AttributeDefinitions"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-attributedef"""
    p_BillingMode: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "BillingMode"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-billingmode"""
    p_ContributorInsightsSpecification: typing.Union['TableContributorInsightsSpecification', dict] = attr.ib(
        default=None,
        converter=TableContributorInsightsSpecification.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(TableContributorInsightsSpecification)),
        metadata={AttrMeta.PROPERTY_NAME: "ContributorInsightsSpecification"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-contributorinsightsspecification-enabled"""
    p_GlobalSecondaryIndexes: typing.List[typing.Union['TableGlobalSecondaryIndex', dict]] = attr.ib(
        default=None,
        converter=TableGlobalSecondaryIndex.from_list,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(TableGlobalSecondaryIndex), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "GlobalSecondaryIndexes"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-gsi"""
    p_KinesisStreamSpecification: typing.Union['TableKinesisStreamSpecification', dict] = attr.ib(
        default=None,
        converter=TableKinesisStreamSpecification.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(TableKinesisStreamSpecification)),
        metadata={AttrMeta.PROPERTY_NAME: "KinesisStreamSpecification"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-kinesisstreamspecification"""
    p_LocalSecondaryIndexes: typing.List[typing.Union['TableLocalSecondaryIndex', dict]] = attr.ib(
        default=None,
        converter=TableLocalSecondaryIndex.from_list,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(TableLocalSecondaryIndex), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "LocalSecondaryIndexes"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-lsi"""
    p_PointInTimeRecoverySpecification: typing.Union['TablePointInTimeRecoverySpecification', dict] = attr.ib(
        default=None,
        converter=TablePointInTimeRecoverySpecification.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(TablePointInTimeRecoverySpecification)),
        metadata={AttrMeta.PROPERTY_NAME: "PointInTimeRecoverySpecification"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-pointintimerecoveryspecification"""
    p_ProvisionedThroughput: typing.Union['TableProvisionedThroughput', dict] = attr.ib(
        default=None,
        converter=TableProvisionedThroughput.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(TableProvisionedThroughput)),
        metadata={AttrMeta.PROPERTY_NAME: "ProvisionedThroughput"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-provisionedthroughput"""
    p_SSESpecification: typing.Union['TableSSESpecification', dict] = attr.ib(
        default=None,
        converter=TableSSESpecification.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(TableSSESpecification)),
        metadata={AttrMeta.PROPERTY_NAME: "SSESpecification"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-ssespecification"""
    p_StreamSpecification: typing.Union['TableStreamSpecification', dict] = attr.ib(
        default=None,
        converter=TableStreamSpecification.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(TableStreamSpecification)),
        metadata={AttrMeta.PROPERTY_NAME: "StreamSpecification"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-streamspecification"""
    p_TableName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "TableName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-tablename"""
    p_TimeToLiveSpecification: typing.Union['TableTimeToLiveSpecification', dict] = attr.ib(
        default=None,
        converter=TableTimeToLiveSpecification.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(TableTimeToLiveSpecification)),
        metadata={AttrMeta.PROPERTY_NAME: "TimeToLiveSpecification"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-timetolivespecification"""
    p_Tags: typing.List[typing.Union[Tag, dict]] = attr.ib(
        default=None,
        converter=Tag.from_list,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(Tag), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "Tags"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#cfn-dynamodb-table-tags"""

    
    @property
    def rv_Arn(self) -> GetAtt:
        """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#aws-resource-dynamodb-table-return-values"""
        return GetAtt(resource=self, attr_name="Arn")
    
    @property
    def rv_StreamArn(self) -> GetAtt:
        """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-table.html#aws-resource-dynamodb-table-return-values"""
        return GetAtt(resource=self, attr_name="StreamArn")
    

@attr.s
class GlobalTable(Resource):
    """
    AWS Object Type = "AWS::DynamoDB::GlobalTable"

    Resource Document: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html

    Property Document:
    
    - ``rp_AttributeDefinitions``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-attributedefinitions
    - ``rp_KeySchema``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-keyschema
    - ``rp_Replicas``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-replicas
    - ``p_BillingMode``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-billingmode
    - ``p_GlobalSecondaryIndexes``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-globalsecondaryindexes
    - ``p_LocalSecondaryIndexes``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-localsecondaryindexes
    - ``p_SSESpecification``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-ssespecification
    - ``p_StreamSpecification``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-streamspecification
    - ``p_TableName``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-tablename
    - ``p_TimeToLiveSpecification``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-timetolivespecification
    - ``p_WriteProvisionedThroughputSettings``: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-writeprovisionedthroughputsettings
    """
    AWS_OBJECT_TYPE = "AWS::DynamoDB::GlobalTable"

    
    rp_AttributeDefinitions: typing.List[typing.Union['GlobalTableAttributeDefinition', dict]] = attr.ib(
        default=None,
        converter=GlobalTableAttributeDefinition.from_list,
        validator=attr.validators.deep_iterable(member_validator=attr.validators.instance_of(GlobalTableAttributeDefinition), iterable_validator=attr.validators.instance_of(list)),
        metadata={AttrMeta.PROPERTY_NAME: "AttributeDefinitions"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-attributedefinitions"""
    rp_KeySchema: typing.List[typing.Union['GlobalTableKeySchema', dict]] = attr.ib(
        default=None,
        converter=GlobalTableKeySchema.from_list,
        validator=attr.validators.deep_iterable(member_validator=attr.validators.instance_of(GlobalTableKeySchema), iterable_validator=attr.validators.instance_of(list)),
        metadata={AttrMeta.PROPERTY_NAME: "KeySchema"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-keyschema"""
    rp_Replicas: typing.List[typing.Union['GlobalTableReplicaSpecification', dict]] = attr.ib(
        default=None,
        converter=GlobalTableReplicaSpecification.from_list,
        validator=attr.validators.deep_iterable(member_validator=attr.validators.instance_of(GlobalTableReplicaSpecification), iterable_validator=attr.validators.instance_of(list)),
        metadata={AttrMeta.PROPERTY_NAME: "Replicas"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-replicas"""
    p_BillingMode: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "BillingMode"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-billingmode"""
    p_GlobalSecondaryIndexes: typing.List[typing.Union['GlobalTableGlobalSecondaryIndex', dict]] = attr.ib(
        default=None,
        converter=GlobalTableGlobalSecondaryIndex.from_list,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(GlobalTableGlobalSecondaryIndex), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "GlobalSecondaryIndexes"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-globalsecondaryindexes"""
    p_LocalSecondaryIndexes: typing.List[typing.Union['GlobalTableLocalSecondaryIndex', dict]] = attr.ib(
        default=None,
        converter=GlobalTableLocalSecondaryIndex.from_list,
        validator=attr.validators.optional(attr.validators.deep_iterable(member_validator=attr.validators.instance_of(GlobalTableLocalSecondaryIndex), iterable_validator=attr.validators.instance_of(list))),
        metadata={AttrMeta.PROPERTY_NAME: "LocalSecondaryIndexes"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-localsecondaryindexes"""
    p_SSESpecification: typing.Union['GlobalTableSSESpecification', dict] = attr.ib(
        default=None,
        converter=GlobalTableSSESpecification.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(GlobalTableSSESpecification)),
        metadata={AttrMeta.PROPERTY_NAME: "SSESpecification"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-ssespecification"""
    p_StreamSpecification: typing.Union['GlobalTableStreamSpecification', dict] = attr.ib(
        default=None,
        converter=GlobalTableStreamSpecification.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(GlobalTableStreamSpecification)),
        metadata={AttrMeta.PROPERTY_NAME: "StreamSpecification"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-streamspecification"""
    p_TableName: TypeHint.intrinsic_str = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(TypeCheck.intrinsic_str_type)),
        metadata={AttrMeta.PROPERTY_NAME: "TableName"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-tablename"""
    p_TimeToLiveSpecification: typing.Union['GlobalTableTimeToLiveSpecification', dict] = attr.ib(
        default=None,
        converter=GlobalTableTimeToLiveSpecification.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(GlobalTableTimeToLiveSpecification)),
        metadata={AttrMeta.PROPERTY_NAME: "TimeToLiveSpecification"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-timetolivespecification"""
    p_WriteProvisionedThroughputSettings: typing.Union['GlobalTableWriteProvisionedThroughputSettings', dict] = attr.ib(
        default=None,
        converter=GlobalTableWriteProvisionedThroughputSettings.from_dict,
        validator=attr.validators.optional(attr.validators.instance_of(GlobalTableWriteProvisionedThroughputSettings)),
        metadata={AttrMeta.PROPERTY_NAME: "WriteProvisionedThroughputSettings"},
    )
    """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#cfn-dynamodb-globaltable-writeprovisionedthroughputsettings"""

    
    @property
    def rv_Arn(self) -> GetAtt:
        """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#aws-resource-dynamodb-globaltable-return-values"""
        return GetAtt(resource=self, attr_name="Arn")
    
    @property
    def rv_StreamArn(self) -> GetAtt:
        """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#aws-resource-dynamodb-globaltable-return-values"""
        return GetAtt(resource=self, attr_name="StreamArn")
    
    @property
    def rv_TableId(self) -> GetAtt:
        """Doc: http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-dynamodb-globaltable.html#aws-resource-dynamodb-globaltable-return-values"""
        return GetAtt(resource=self, attr_name="TableId")
    
